/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ide.ui;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.internal.Utils;
import org.eclipse.target.internal.ide.IdeTargetResource;
import org.eclipse.target.internal.ide.IdeWorkspaceSite;
import org.eclipse.target.internal.ui.actions.CopyOperation;
import org.eclipse.target.internal.ui.dnd.TargetResourceTransfer;
import org.eclipse.ui.part.IDropActionDelegate;

/**
 * @author Administrator
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class TargetResourceDropActionDelegate implements IDropActionDelegate {

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.IDropActionDelegate#run(java.lang.Object, java.lang.Object)
	 */
	public boolean run(Object source, Object target) {
		if (target instanceof IContainer) {
			TargetResourceTransfer transfer = TargetResourceTransfer.getInstance();
			ITargetResource[] toDrop = transfer.fromByteArray((byte[]) source);
			if (toDrop == null) return false;
			ITargetResource parent = IdeTargetResource.createResource(IdeWorkspaceSite.getInstance(), (IContainer) target);
			//cannot drop a resource onto itself or a child
			for (int i = 0; i < toDrop.length; i++)
				if (toDrop[i].equals(target) || toDrop[i].getFullPath().startsWith(parent.getFullPath()))
					return false;
			try {
				// Run the copy in the foreground
				new CopyOperation(null, parent, toDrop) {
					protected boolean canRunAsJob() {
						return false;
					}
				}.run();
			} catch (InvocationTargetException e) {
				Utils.handle(null, e);
				return false;
			} catch (InterruptedException e) {
				return false;
			}
			return true;
		}
		//drop was not successful so return false
		return false;

	}

}
