/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.internal.Policy;
import org.eclipse.target.ui.TargetOperation;
import org.eclipse.ui.IWorkbenchPart;

/**
 * An operation that copies resources from one site to another.
 */
public class CopyOperation extends TargetOperation {

	private ITargetResource[] resources;
	private ITargetResource targetFolder;

	public CopyOperation(IWorkbenchPart part, ITargetResource targetFolder, ITargetResource[] resources) {
		super(part);
		this.targetFolder = targetFolder;
		this.resources = resources;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.ui.TargetOperation#canRunAsJob()
	 */
	protected boolean canRunAsJob() {
		return true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
		try {
			performCopy(resources, targetFolder, monitor);
		} catch (CoreException e) {
			throw new InvocationTargetException(e);
		}
	}

	protected void performCopy(ITargetResource[] resources, ITargetResource selectedFolder, IProgressMonitor monitor) throws CoreException {
		monitor.beginTask("Copying resources", 100 * resources.length);
		for (int i = 0; i < resources.length; i++) {
			ITargetResource resource = resources[i];
			performCopy(resource, selectedFolder, Policy.subMonitorFor(monitor, 100));
		}	
	}
	
	private void performCopy(ITargetResource source, ITargetResource parentFolder, IProgressMonitor monitor) throws CoreException {
		if (source.isDirectory()) {
			monitor.beginTask(null, 100);
			ITargetResource target = parentFolder.getFolder(source.getName());
			target.mkdir(Policy.subMonitorFor(monitor, 10));
			ITargetResource[] members = source.members(Policy.subMonitorFor(monitor, 10));
			performCopy(members, target, Policy.subMonitorFor(monitor, 80));
			monitor.done();
		} else {
			monitor.beginTask(null, 100);
			ITargetResource target = parentFolder.getFile(source.getName());
			source.fetchProperties(new QualifiedName[] {ITargetResource.SIZE_PROPERTY}, false, Policy.subMonitorFor(monitor, 10));
			target.setContents(source.getContents(Policy.subMonitorFor(monitor, 40)), target.getSize(), Policy.subMonitorFor(monitor, 50));
			monitor.done();
		}
	}
}
